#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

import argparse

import openstack.config
from openstack.tests.unit.config import base


class TestInit(base.TestCase):
    def test_get_cloud_region_without_arg_parser(self):
        cloud_region = openstack.config.get_cloud_region(
            options=None, validate=False)
        self.assertIsInstance(
            cloud_region,
            openstack.config.cloud_region.CloudRegion
        )

    def test_get_cloud_region_with_arg_parser(self):
        cloud_region = openstack.config.get_cloud_region(
            options=argparse.ArgumentParser(),
            validate=False)
        self.assertIsInstance(
            cloud_region,
            openstack.config.cloud_region.CloudRegion
        )
